@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2009 Dale Whittaker <dayul@users.sf.net>
+ Copyright (C) 2009-2010 Dale Whittaker <dayul@users.sf.net>
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
@@ -91,7 +91,7 @@ feed_panel_treeview_render_icon_cb (GtkTreeViewColumn* column,
     uri = katze_item_get_uri (pitem);
     if (uri)
     {
-        pixbuf = katze_net_load_icon (panel->net, uri, NULL, NULL, NULL);
+        pixbuf = katze_load_cached_icon (uri, NULL);
         if (!pixbuf)
             pixbuf = panel->pixbuf;
     }
@@ -553,8 +553,8 @@ feed_panel_popup (GtkWidget*      widget,
             item, feed_panel_delete_activate_cb, panel);
     }
 
-    sokoke_widget_popup (widget, GTK_MENU (menu),
-                         event, SOKOKE_MENU_POSITION_CURSOR);
+    katze_widget_popup (widget, GTK_MENU (menu),
+                        event, KATZE_MENU_POSITION_CURSOR);
 }
 
 static gboolean
@@ -657,8 +657,12 @@ webview_navigation_request_cb (WebKitWebView*             web_view,
         uri = webkit_network_request_get_uri (request);
         n = midori_browser_add_uri (browser, uri);
         midori_browser_set_current_page (browser, n);
+        webkit_web_policy_decision_ignore (policy_decision);
+
+        return TRUE;
     }
-    return TRUE;
+
+    return FALSE;
 }
 
 static const gchar*
