@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2008-2009 Christian Dywan <christian@twotoasts.de>
+ Copyright (C) 2008-2010 Christian Dywan <christian@twotoasts.de>
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
@@ -76,7 +76,7 @@ _katze_array_add_item (KatzeArray* array,
     {
         GType type = G_OBJECT_TYPE (item);
 
-        g_return_if_fail (katze_array_is_a (array, type));
+        /* g_return_if_fail (katze_array_is_a (array, type)); */
         g_object_ref (item);
         if (g_type_is_a (type, KATZE_TYPE_ITEM))
             katze_item_set_parent (item, array);
@@ -166,7 +166,7 @@ katze_array_class_init (KatzeArrayClass* class)
         G_STRUCT_OFFSET (KatzeArrayClass, move_item),
         0,
         NULL,
-        katze_cclosure_marshal_VOID__POINTER_INT,
+        midori_cclosure_marshal_VOID__POINTER_INT,
         G_TYPE_NONE, 2,
         G_TYPE_POINTER,
         G_TYPE_INT);
@@ -375,13 +375,56 @@ katze_array_find_token (KatzeArray*  array,
     i = 0;
     while ((item = g_list_nth_data (array->items, i++)))
     {
-        const gchar* found_token = katze_item_get_token ((KatzeItem*)item);
+        const gchar* found_token;
+
+        if (!KATZE_IS_ITEM (item))
+            continue;
+        found_token = ((KatzeItem*)item)->token;
         if (!g_strcmp0 (found_token, token))
             return item;
     }
     return NULL;
 }
 
+/**
+ * katze_array_find_uri:
+ * @array: a #KatzeArray
+ * @uri: an URI
+ *
+ * Looks up an item in the array which has the specified URI.
+ *
+ * This function will silently fail if the type of the list
+ * is not based on #GObject and only #KatzeItem children
+ * are checked for their token, any other objects are skipped.
+ *
+ * Return value: an item, or %NULL
+ *
+ * Since: 0.2.0
+ **/
+gpointer
+katze_array_find_uri (KatzeArray*  array,
+                      const gchar* uri)
+{
+    guint i;
+    gpointer item;
+
+    if (!katze_array_is_a (array, G_TYPE_OBJECT))
+        return NULL;
+
+    i = 0;
+    while ((item = g_list_nth_data (array->items, i++)))
+    {
+        const gchar* found_uri;
+
+        if (!KATZE_IS_ITEM (item))
+            continue;
+        found_uri = ((KatzeItem*)item)->uri;
+        if (!g_strcmp0 (found_uri, uri))
+            return item;
+    }
+    return NULL;
+}
+
 /**
  * katze_array_get_length:
  * @array: a #KatzeArray
@@ -418,6 +461,24 @@ katze_array_move_item (KatzeArray* array,
     g_signal_emit (array, signals[MOVE_ITEM], 0, item, position);
 }
 
+/**
+ * katze_array_get_items:
+ * @array: a #KatzeArray
+ *
+ * Retrieves the items as a list.
+ *
+ * Return value: a newly allocated #GList of items
+ *
+ * Since: 0.2.5
+ **/
+GList*
+katze_array_get_items (KatzeArray* array)
+{
+    g_return_val_if_fail (KATZE_IS_ARRAY (array), NULL);
+
+    return g_list_copy (array->items);
+}
+
 /**
  * katze_array_clear:
  * @array: a #KatzeArray
