@@ -39,9 +39,9 @@ struct _KatzeItem
     gchar* name;
     gchar* text;
     gchar* uri;
-    gchar* icon;
     gchar* token;
     gint64 added;
+    GHashTable* metadata;
 
     KatzeItem* parent;
 };
@@ -55,7 +55,7 @@ struct _KatzeItemClass
 };
 
 GType
-katze_item_get_type               (void);
+katze_item_get_type               (void) G_GNUC_CONST;
 
 KatzeItem*
 katze_item_new                    (void);
@@ -102,6 +102,27 @@ void
 katze_item_set_added              (KatzeItem*      item,
                                    gint64          added);
 
+GList*
+katze_item_get_meta_keys          (KatzeItem*      item);
+
+const gchar*
+katze_item_get_meta_string        (KatzeItem*      item,
+                                   const gchar*    key);
+
+void
+katze_item_set_meta_string        (KatzeItem*      item,
+                                   const gchar*    key,
+                                   const gchar*    value);
+
+gint64
+katze_item_get_meta_integer       (KatzeItem*      item,
+                                   const gchar*    key);
+
+void
+katze_item_set_meta_integer       (KatzeItem*      item,
+                                   const gchar*    key,
+                                   gint64          value);
+
 gpointer
 katze_item_get_parent             (KatzeItem*      item);
 
