@@ -39,12 +39,14 @@ G_BEGIN_DECLS
     #define GTK_TYPE_ICON_ENTRY GTK_TYPE_ENTRY
     #define gtk_icon_entry_new gtk_entry_new
     #define gtk_icon_entry_set_icon_from_stock gtk_entry_set_icon_from_stock
+    #define gtk_icon_entry_set_icon_from_icon_name gtk_entry_set_icon_from_icon_name
 
     void
     gtk_icon_entry_set_icon_from_pixbuf (GtkEntry*            entry,
                                          GtkEntryIconPosition position,
                                          GdkPixbuf*           pixbuf);
     #define gtk_icon_entry_set_icon_highlight gtk_entry_set_icon_activatable
+    #define gtk_icon_entry_set_progress_fraction gtk_entry_set_progress_fraction
 #else
 
 #define GTK_TYPE_ICON_ENTRY (gtk_icon_entry_get_type())
@@ -60,12 +62,15 @@ typedef enum
   GTK_ICON_ENTRY_SECONDARY
 } GtkIconEntryPosition;
 
-typedef struct _GtkIconEntry      GtkIconEntry;
-typedef struct _GtkIconEntryClass GtkIconEntryClass;
+typedef struct _GtkIconEntry        GtkIconEntry;
+typedef struct _GtkIconEntryClass   GtkIconEntryClass;
+typedef struct _GtkIconEntryPrivate GtkIconEntryPrivate;
 
 struct _GtkIconEntry
 {
   GtkEntry parent_object;
+
+  GtkIconEntryPrivate* priv;
 };
 
 struct _GtkIconEntryClass
@@ -129,6 +134,9 @@ void       gtk_icon_entry_set_icon_sensitive      (const GtkIconEntry *icon_entr
 						   GtkIconEntryPosition icon_pos,
 						   gboolean sensitive);
 
+void       gtk_icon_entry_set_progress_fraction    (GtkIconEntry *icon_entry,
+                                                    gdouble       fraction);
+
 #endif
 
 G_END_DECLS
