@@ -36,7 +36,7 @@ typedef struct _MidoriApp                MidoriApp;
 typedef struct _MidoriAppClass           MidoriAppClass;
 
 GType
-midori_app_get_type               (void);
+midori_app_get_type               (void) G_GNUC_CONST;
 
 MidoriApp*
 midori_app_new                    (void);
@@ -54,6 +54,10 @@ gboolean
 midori_app_instance_send_uris     (MidoriApp*         app,
                                    gchar**            uris);
 
+gboolean
+midori_app_send_command           (MidoriApp*         app,
+                                   gchar**            command);
+
 void
 midori_app_add_browser            (MidoriApp*         app,
                                    MidoriBrowser*     browser);
@@ -61,6 +65,12 @@ midori_app_add_browser            (MidoriApp*         app,
 MidoriBrowser*
 midori_app_create_browser         (MidoriApp*         app);
 
+MidoriBrowser*
+midori_app_get_browser            (MidoriApp*         app);
+
+GList*
+midori_app_get_browsers           (MidoriApp*         app);
+
 void
 midori_app_quit                   (MidoriApp*         app);
 
