@@ -1,6 +1,6 @@
 /*
- Copyright (C) 2008-2009 Christian Dywan <christian@twotoasts.de>
- Copyright (C) 2008 Dale Whittaker <dayul@users.sf.net>
+ Copyright (C) 2008-2010 Christian Dywan <christian@twotoasts.de>
+ Copyright (C) 2008-2010 Dale Whittaker <dayul@users.sf.net>
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
@@ -87,6 +87,7 @@ enum
     VISIBLE_COL,
     YALIGN_COL,
     BACKGROUND_COL,
+    STYLE_COL,
     N_COLS
 };
 
@@ -268,7 +269,8 @@ midori_location_action_create_model (void)
 {
     GtkTreeModel* model = (GtkTreeModel*) gtk_list_store_new (N_COLS,
         GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING,
-        G_TYPE_INT, G_TYPE_BOOLEAN, G_TYPE_FLOAT, GDK_TYPE_COLOR);
+        G_TYPE_INT, G_TYPE_BOOLEAN, G_TYPE_FLOAT,
+        GDK_TYPE_COLOR, G_TYPE_BOOLEAN);
     return model;
 }
 
@@ -340,9 +342,11 @@ midori_location_action_popup_timeout_cb (gpointer data)
     gint result;
     static sqlite3_stmt* stmt;
     const gchar* sqlcmd;
-    gint matches, searches, height, screen_height, sep;
+    gint matches, searches, height, screen_height, browser_height, sep;
+    MidoriBrowser* browser;
+    GtkStyle* style;
 
-    if (!gtk_widget_has_focus (action->entry) || !action->history)
+    if (!action->entry || !gtk_widget_has_focus (action->entry) || !action->history)
         return FALSE;
 
     if (!(action->key && *action->key))
@@ -355,18 +359,22 @@ midori_location_action_popup_timeout_cb (gpointer data)
     {
         sqlite3* db;
         db = g_object_get_data (G_OBJECT (action->history), "db");
-        sqlcmd = "SELECT uri, title FROM history WHERE uri LIKE ? OR title LIKE ?"
-                 " GROUP BY uri ORDER BY count() DESC LIMIT ?";
-        sqlite3_prepare_v2 (db, sqlcmd, -1, &stmt, NULL);
+        sqlcmd = "SELECT type, uri, title, count() AS ct FROM history_view "
+                 "WHERE uri LIKE ?1 OR title LIKE ?1 GROUP BY uri "
+                 "UNION ALL "
+                 "SELECT type, replace(uri, '%s', title) AS uri, title, count() AS ct FROM search_view "
+                 "WHERE title LIKE ?1 GROUP BY uri "
+                 "ORDER BY ct DESC LIMIT ?2";
+        sqlite3_prepare_v2 (db, sqlcmd, strlen (sqlcmd) + 1, &stmt, NULL);
     }
     sqlite3_bind_text (stmt, 1, g_strdup_printf ("%%%s%%", action->key), -1, g_free);
-    sqlite3_bind_text (stmt, 2, g_strdup_printf ("%%%s%%", action->key), -1, g_free);
-    sqlite3_bind_int64 (stmt, 3, MAX_ITEMS);
+    sqlite3_bind_int64 (stmt, 2, MAX_ITEMS);
 
     result = sqlite3_step (stmt);
     if (result != SQLITE_ROW && !action->search_engines)
     {
-        g_print (_("Failed to select from history\n"));
+        if (result == SQLITE_ERROR)
+            g_print (_("Failed to select from history\n"));
         sqlite3_reset (stmt);
         sqlite3_clear_bindings (stmt);
         midori_location_action_popdown_completion (action);
@@ -377,6 +385,7 @@ midori_location_action_popup_timeout_cb (gpointer data)
     {
         GtkTreeModel* model = NULL;
         GtkWidget* popup;
+        GtkWidget* popup_frame;
         GtkWidget* scrolled;
         GtkWidget* treeview;
         GtkCellRenderer* renderer;
@@ -386,10 +395,13 @@ midori_location_action_popup_timeout_cb (gpointer data)
 
         popup = gtk_window_new (GTK_WINDOW_POPUP);
         gtk_window_set_type_hint (GTK_WINDOW (popup), GDK_WINDOW_TYPE_HINT_COMBO);
+        popup_frame = gtk_frame_new (NULL);
+        gtk_frame_set_shadow_type (GTK_FRAME (popup_frame), GTK_SHADOW_ETCHED_IN);
+        gtk_container_add (GTK_CONTAINER (popup), popup_frame);
         scrolled = g_object_new (GTK_TYPE_SCROLLED_WINDOW,
             "hscrollbar-policy", GTK_POLICY_NEVER,
             "vscrollbar-policy", GTK_POLICY_AUTOMATIC, NULL);
-        gtk_container_add (GTK_CONTAINER (popup), scrolled);
+        gtk_container_add (GTK_CONTAINER (popup_frame), scrolled);
         treeview = gtk_tree_view_new_with_model (model);
         gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);
         gtk_tree_view_set_hover_selection (GTK_TREE_VIEW (treeview), TRUE);
@@ -429,16 +441,28 @@ midori_location_action_popup_timeout_cb (gpointer data)
     gtk_list_store_clear (store);
 
     matches = searches = 0;
+    style = gtk_widget_get_style (action->treeview);
     while (result == SQLITE_ROW)
     {
-        const unsigned char* uri = sqlite3_column_text (stmt, 0);
-        const unsigned char* title = sqlite3_column_text (stmt, 1);
+        sqlite3_int64 type = sqlite3_column_int64 (stmt, 0);
+        const unsigned char* uri = sqlite3_column_text (stmt, 1);
+        const unsigned char* title = sqlite3_column_text (stmt, 2);
         GdkPixbuf* icon = katze_load_cached_icon ((gchar*)uri, NULL);
         if (!icon)
             icon = action->default_icon;
-        gtk_list_store_insert_with_values (store, NULL, matches,
-            URI_COL, uri, TITLE_COL, title, YALIGN_COL, 0.25,
-            FAVICON_COL, icon, -1);
+        if (type == 1 /* history_view */)
+            gtk_list_store_insert_with_values (store, NULL, matches,
+                URI_COL, uri, TITLE_COL, title, YALIGN_COL, 0.25,
+                FAVICON_COL, icon, -1);
+        else if (type == 2 /* search_view */)
+        {
+            gchar* search_title = g_strdup_printf (_("Search for %s"), title);
+            gtk_list_store_insert_with_values (store, NULL, matches,
+                URI_COL, uri, TITLE_COL, search_title, YALIGN_COL, 0.25,
+                STYLE_COL, 1, FAVICON_COL, icon, -1);
+            g_free (search_title);
+        }
+
         matches++;
         result = sqlite3_step (stmt);
     }
@@ -453,15 +477,13 @@ midori_location_action_popup_timeout_cb (gpointer data)
         {
             gchar* uri;
             gchar* title;
-            GtkStyle* style;
 
             uri = sokoke_search_uri (katze_item_get_uri (item), action->key);
             title = g_strdup_printf (_("Search with %s"), katze_item_get_name (item));
-            style = gtk_widget_get_style (action->treeview);
             gtk_list_store_insert_with_values (store, NULL, matches + i,
                 URI_COL, uri, TITLE_COL, title, YALIGN_COL, 0.25,
                 BACKGROUND_COL, style ? &style->bg[GTK_STATE_NORMAL] : NULL,
-                FAVICON_COL, NULL, -1);
+                STYLE_COL, 1, FAVICON_COL, NULL, -1);
             g_free (uri);
             g_free (title);
             i++;
@@ -469,7 +491,7 @@ midori_location_action_popup_timeout_cb (gpointer data)
         searches += i;
     }
 
-    if (!GTK_WIDGET_VISIBLE (action->popup))
+    if (!gtk_widget_get_visible (action->popup))
     {
         GtkWidget* toplevel = gtk_widget_get_toplevel (action->entry);
         gtk_window_set_screen (GTK_WINDOW (action->popup),
@@ -478,13 +500,16 @@ midori_location_action_popup_timeout_cb (gpointer data)
         gtk_tree_view_columns_autosize (GTK_TREE_VIEW (action->treeview));
     }
 
+    browser = midori_browser_get_for_widget (action->entry);
     column = gtk_tree_view_get_column (GTK_TREE_VIEW (action->treeview), 0);
     gtk_tree_view_column_cell_get_size (column, NULL, NULL, NULL, NULL, &height);
     screen_height = gdk_screen_get_height (gtk_widget_get_screen (action->popup));
+    gtk_window_get_size (GTK_WINDOW (browser), NULL, &browser_height);
+    screen_height = MIN (MIN (browser_height, screen_height / 1.5), screen_height / 1.5);
     gtk_widget_style_get (action->treeview, "vertical-separator", &sep, NULL);
     /* FIXME: Instead of 1.5 we should relate to the height of one line */
     height = MIN (matches * height + (matches + searches) * sep
-                                   + searches * height / 1.5, screen_height / 1.5);
+                                   + searches * height / 1.5, screen_height);
     gtk_widget_set_size_request (action->treeview, -1, height);
     midori_location_action_popup_position (action->popup, action->entry);
     gtk_widget_show_all (action->popup);
@@ -811,7 +836,7 @@ midori_location_action_button_press_event_cb (GtkEntry*             entry,
                                               GdkEventKey*          event,
                                               MidoriLocationAction* action)
 {
-    if (action->popup && GTK_WIDGET_VISIBLE (action->popup))
+    if (action->popup && gtk_widget_get_visible (action->popup))
     {
         midori_location_action_popdown_completion (action);
 
@@ -843,7 +868,7 @@ midori_location_action_key_press_event_cb (GtkEntry*    entry,
     case GDK_Right:
     case GDK_KP_Right:
 
-        if (location_action->popup && GTK_WIDGET_VISIBLE (location_action->popup))
+        if (location_action->popup && gtk_widget_get_visible (location_action->popup))
         {
             GtkTreeModel* model = location_action->completion_model;
             GtkTreeIter iter;
@@ -872,7 +897,7 @@ midori_location_action_key_press_event_cb (GtkEntry*    entry,
         break;
     case GDK_Escape:
     {
-        if (location_action->popup && GTK_WIDGET_VISIBLE (location_action->popup))
+        if (location_action->popup && gtk_widget_get_visible (location_action->popup))
         {
             midori_location_action_popdown_completion (location_action);
             text = gtk_entry_get_text (entry);
@@ -886,7 +911,7 @@ midori_location_action_key_press_event_cb (GtkEntry*    entry,
     }
     case GDK_Page_Up:
     case GDK_Page_Down:
-        if (!(location_action->popup && GTK_WIDGET_VISIBLE (location_action->popup)))
+        if (!(location_action->popup && gtk_widget_get_visible (location_action->popup)))
             return TRUE;
     case GDK_Down:
     case GDK_KP_Down:
@@ -895,7 +920,7 @@ midori_location_action_key_press_event_cb (GtkEntry*    entry,
     {
         GtkWidget* parent;
 
-        if (location_action->popup && GTK_WIDGET_VISIBLE (location_action->popup))
+        if (location_action->popup && gtk_widget_get_visible (location_action->popup))
         {
             GtkTreeModel* model = location_action->completion_model;
             gint matches = gtk_tree_model_iter_n_children (model, NULL);
@@ -1018,6 +1043,7 @@ midori_location_entry_render_text_cb (GtkCellLayout*   layout,
     gchar* uri;
     gchar* title;
     GdkColor* background;
+    gboolean style;
     gchar* desc;
     gchar* desc_uri;
     gchar* desc_title;
@@ -1030,9 +1056,9 @@ midori_location_entry_render_text_cb (GtkCellLayout*   layout,
     size_t len;
 
     gtk_tree_model_get (model, iter, URI_COL, &uri, TITLE_COL, &title,
-        BACKGROUND_COL, &background, -1);
+        BACKGROUND_COL, &background, STYLE_COL, &style, -1);
 
-    if (background != NULL) /* A search engine action */
+    if (style) /* A search engine action */
     {
         g_object_set (renderer, "text", title,
             "ellipsize-set", TRUE, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
@@ -1659,3 +1685,60 @@ midori_location_action_clear (MidoriLocationAction* location_action)
 
     midori_location_action_toggle_arrow (location_action);
 }
+
+/**
+ * midori_location_action_set_security_hint:
+ * @location_action: a #MidoriLocationAction
+ * @hint: a security hint
+ *
+ * Sets a security hint on the action, so that the security status
+ * can be reflected visually.
+ *
+ * Since: 0.2.5
+ **/
+void
+midori_location_action_set_security_hint (MidoriLocationAction* location_action,
+                                          MidoriSecurity        hint)
+{
+    GSList* proxies;
+    GtkWidget* entry;
+    GtkWidget* child;
+
+    g_return_if_fail (MIDORI_IS_LOCATION_ACTION (location_action));
+
+    proxies = gtk_action_get_proxies (GTK_ACTION (location_action));
+
+    for (; proxies != NULL; proxies = g_slist_next (proxies))
+    if (GTK_IS_TOOL_ITEM (proxies->data))
+    {
+        GdkColor bg_color = { 0, 1 };
+        GdkColor fg_color = { 0, 1 };
+
+        entry = midori_location_action_entry_for_proxy (proxies->data);
+        child = gtk_bin_get_child (GTK_BIN (entry));
+
+        if (hint == MIDORI_SECURITY_UNKNOWN)
+        {
+            gdk_color_parse ("#ef7070", &bg_color);
+            gdk_color_parse ("#000", &fg_color);
+            #if !HAVE_HILDON
+            gtk_icon_entry_set_icon_from_stock (GTK_ICON_ENTRY (child),
+                GTK_ICON_ENTRY_SECONDARY, GTK_STOCK_INFO);
+            #endif
+        }
+        else if (hint == MIDORI_SECURITY_TRUSTED)
+        {
+            gdk_color_parse ("#fcf19a", &bg_color);
+            gdk_color_parse ("#000", &fg_color);
+            #if !HAVE_HILDON
+            gtk_icon_entry_set_icon_from_stock (GTK_ICON_ENTRY (child),
+                GTK_ICON_ENTRY_SECONDARY, GTK_STOCK_DIALOG_AUTHENTICATION);
+            #endif
+        }
+
+        gtk_widget_modify_base (child, GTK_STATE_NORMAL,
+            bg_color.red == 1 ? NULL : &bg_color);
+        gtk_widget_modify_text (child, GTK_STATE_NORMAL,
+            bg_color.red == 1 ? NULL : &fg_color);
+    }
+}
