@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2008 Christian Dywan <christian@twotoasts.de>
+ Copyright (C) 2008-2009 Christian Dywan <christian@twotoasts.de>
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
@@ -12,9 +12,8 @@
 #ifndef __MIDORI_LOCATION_ACTION_H__
 #define __MIDORI_LOCATION_ACTION_H__
 
-#include "midori-locationentry.h"
-
 #include <katze/katze.h>
+#include "midori-view.h"
 
 G_BEGIN_DECLS
 
@@ -34,18 +33,24 @@ G_BEGIN_DECLS
 typedef struct _MidoriLocationAction         MidoriLocationAction;
 typedef struct _MidoriLocationActionClass    MidoriLocationActionClass;
 
-struct _MidoriLocationEntryItem
-{
-    GdkPixbuf* favicon;
-    const gchar* uri;
-    const gchar* title;
-};
-
 GType
-midori_location_action_get_type             (void);
+midori_location_action_get_type             (void) G_GNUC_CONST;
+
+void
+midori_location_action_freeze              (MidoriLocationAction* location_action);
+
+void
+midori_location_action_thaw                (MidoriLocationAction* location_action);
 
 const gchar*
-midori_location_action_get_uri              (MidoriLocationAction* action);
+midori_location_action_get_text             (MidoriLocationAction* location_action);
+
+void
+midori_location_action_set_text             (MidoriLocationAction* location_action,
+                                             const gchar*          text);
+
+const gchar*
+midori_location_action_get_uri              (MidoriLocationAction* location_action);
 
 void
 midori_location_action_set_uri              (MidoriLocationAction* location_action,
@@ -64,6 +69,7 @@ midori_location_action_add_item             (MidoriLocationAction* location_acti
                                              const gchar*          uri,
                                              GdkPixbuf*            icon,
                                              const gchar*          title);
+
 void
 midori_location_action_set_icon_for_uri     (MidoriLocationAction* location_action,
                                              GdkPixbuf*            icon,
@@ -74,6 +80,10 @@ midori_location_action_set_title_for_uri    (MidoriLocationAction* location_acti
                                              const gchar*          title,
                                              const gchar*          text);
 
+void
+midori_location_action_set_search_engines   (MidoriLocationAction* location_action,
+                                             KatzeArray*           search_engines);
+
 gdouble
 midori_location_action_get_progress         (MidoriLocationAction* location_action);
 
@@ -92,6 +102,10 @@ midori_location_action_delete_item_from_uri (MidoriLocationAction* location_acti
 void
 midori_location_action_clear                (MidoriLocationAction* location_action);
 
+void
+midori_location_action_set_security_hint    (MidoriLocationAction* location_action,
+                                             MidoriSecurity        hint);
+
 G_END_DECLS
 
 #endif /* __MIDORI_LOCATION_ACTION_H__ */
