@@ -16,12 +16,15 @@
 #endif
 
 #include "sokoke.h"
-#include "compat.h"
 
 #include <string.h>
 #include <glib/gi18n.h>
 #include <libsoup/soup.h>
 
+#if HAVE_LIBNOTIFY
+    #include <libnotify/notify.h>
+#endif
+
 struct _MidoriPreferences
 {
     KatzePreferences parent_instance;
@@ -180,25 +183,17 @@ midori_preferences_homepage_current_clicked_cb (GtkWidget*         button,
 
 #if !HAVE_HILDON
 static void
-midori_preferences_notify_auto_detect_proxy_cb (MidoriWebSettings* settings,
-                                                GParamSpec*        pspec,
-                                                GtkWidget*         entry)
+midori_preferences_notify_proxy_type_cb (MidoriWebSettings* settings,
+                                         GParamSpec*        pspec,
+                                         GtkWidget*         entry)
 {
-    MidoriIdentity auto_detect_proxy = katze_object_get_enum (settings,
-                                                              "auto-detect-proxy");
+    MidoriProxy proxy_type = katze_object_get_enum (settings, "proxy-type");
 
-    gtk_widget_set_sensitive (entry, !auto_detect_proxy);
+    gtk_widget_set_sensitive (entry, proxy_type == MIDORI_PROXY_HTTP);
 }
 #endif
 
 #if HAVE_OSX
-static void
-midori_preferences_help_clicked_cb (GtkWidget* button,
-                                    GtkDialog* dialog)
-{
-    gtk_dialog_response (dialog, GTK_RESPONSE_HELP);
-}
-
 static void
 midori_preferences_toolbutton_clicked_cb (GtkWidget* toolbutton,
                                           GtkWidget* page)
@@ -227,6 +222,7 @@ midori_preferences_add_toolbutton (GtkWidget*   toolbar,
 #endif
 }
 
+#if 0
 static void
 midori_preferences_list_dicts_cb (const gchar* lang_tag,
                                   const gchar* provider_name,
@@ -240,33 +236,13 @@ midori_preferences_list_dicts_cb (const gchar* lang_tag,
 static GList*
 midori_preferences_get_spell_languages (void)
 {
-    static gpointer (*enchant_broker_init) (void) = NULL;
-    static void (*enchant_broker_list_dicts) (gpointer, GCallback, gpointer) = NULL;
-    static void (*enchant_broker_free) (gpointer) = NULL;
-    gpointer broker;
     GList* dicts = NULL;
-
-    if (!enchant_broker_list_dicts && g_module_supported ())
-    {
-        GModule* module;
-        if (!(module = g_module_open ("libenchant.so", G_MODULE_BIND_LOCAL)))
-            return NULL;
-        if (!g_module_symbol (module, "enchant_broker_init",
-                              (void*) &enchant_broker_init))
-            return NULL;
-        if (!g_module_symbol (module, "enchant_broker_list_dicts",
-                              (void*) &enchant_broker_list_dicts))
-            return NULL;
-        if (!g_module_symbol (module, "enchant_broker_free",
-                              (void*) &enchant_broker_free))
-            return NULL;
-    }
-
-    broker = enchant_broker_init ();
+    gpointer broker = enchant_broker_init ();
     enchant_broker_list_dicts (broker, (GCallback)midori_preferences_list_dicts_cb, &dicts);
     enchant_broker_free (broker);
     return dicts;
 }
+#endif
 
 /**
  * midori_preferences_set_settings:
@@ -289,9 +265,6 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     GtkWidget* label;
     GtkWidget* button;
     GtkWidget* entry;
-    #if WEBKIT_CHECK_VERSION (1, 1, 6)
-    GList* languages;
-    #endif
 
     g_return_if_fail (MIDORI_IS_PREFERENCES (preferences));
     g_return_if_fail (MIDORI_IS_WEB_SETTINGS (settings));
@@ -363,10 +336,13 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     SPANNED_ADD (button);
     label = katze_property_proxy (settings, "ask-for-destination-folder", NULL);
     INDENTED_ADD (label);
-    button = katze_property_proxy (settings, "notify-transfer-completed", NULL);
-    /* FIXME: Disable the option if notifications presumably cannot be sent
-      gtk_widget_set_sensitive (button, FALSE); */
-    SPANNED_ADD (button);
+    #if HAVE_LIBNOTIFY
+    if (notify_is_initted () || g_find_program_in_path ("notify-send"))
+    {
+        button = katze_property_proxy (settings, "notify-transfer-completed", NULL);
+        SPANNED_ADD (button);
+    }
+    #endif
     #endif
 
     /* Page "Appearance" */
@@ -413,9 +389,9 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
         button = katze_property_proxy (settings, "kinetic-scrolling", NULL);
     else
     {
-        button = katze_property_proxy (settings, "auto-shrink-images", NULL);
-        gtk_button_set_label (GTK_BUTTON (button), _("Shrink images automatically"));
-        gtk_widget_set_tooltip_text (button, _("Automatically shrink standalone images to fit"));
+        button = katze_property_proxy (settings, "enforce-96-dpi", NULL);
+        gtk_button_set_label (GTK_BUTTON (button), _("Enforce 96 dots per inch"));
+        gtk_widget_set_tooltip_text (button, _("Enforce a video dot density of 96 DPI"));
     }
     #else
     button = katze_property_proxy (settings, "middle-click-opens-selection", NULL);
@@ -424,36 +400,35 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     #if !HAVE_HILDON
     button = katze_property_proxy (settings, "enable-scripts", NULL);
     INDENTED_ADD (button);
+    button = katze_property_proxy (settings, "open-panels-in-windows", NULL);
+    SPANNED_ADD (button);
     button = katze_property_proxy (settings, "enable-plugins", NULL);
-    SPANNED_ADD (button);
-    button = katze_property_proxy (settings, "enforce-96-dpi", NULL);
-    gtk_button_set_label (GTK_BUTTON (button), _("Enforce 96 dots per inch"));
-    gtk_widget_set_tooltip_text (button, _("Enforce a video dot density of 96 DPI"));
     INDENTED_ADD (button);
-    button = katze_property_proxy (settings, "enable-developer-extras", NULL);
-    gtk_button_set_label (GTK_BUTTON (button), _("Enable developer tools"));
-    gtk_widget_set_tooltip_text (button, _("Enable special extensions for developers"));
-    SPANNED_ADD (button);
     #endif
-    button = katze_property_proxy (settings, "zoom-text-and-images", NULL);
-    INDENTED_ADD (button);
     button = katze_property_proxy (settings, "find-while-typing", NULL);
     SPANNED_ADD (button);
+    button = katze_property_proxy (settings, "zoom-text-and-images", NULL);
+    INDENTED_ADD (button);
+    #if WEBKIT_CHECK_VERSION (1, 1, 11)
+    button = katze_property_proxy (settings, "javascript-can-open-windows-automatically", NULL);
+    gtk_button_set_label (GTK_BUTTON (button), _("Allow scripts to open popups"));
+    gtk_widget_set_tooltip_text (button, _("Whether scripts are allowed to open popup windows automatically"));
+    SPANNED_ADD (button);
+    #endif
     #if WEBKIT_CHECK_VERSION (1, 1, 6)
-    if ((languages = midori_preferences_get_spell_languages ()))
-    {
-        FRAME_NEW (_("Spell Checking"));
-        button = katze_property_proxy (settings, "enable-spell-checking", NULL);
-        gtk_button_set_label (GTK_BUTTON (button), _("Enable Spell Checking"));
-        gtk_widget_set_tooltip_text (button, _("Enable spell checking while typing"));
-        INDENTED_ADD (button);
-        entry = katze_property_proxy (settings, "spell-checking-languages", NULL);
-        /* i18n: The example should be adjusted to contain a good local default */
-        gtk_widget_set_tooltip_text (entry, _("A comma separated list of "
-           "languages to be used for spell checking, for example \"en_GB,de_DE\""));
-        SPANNED_ADD (entry);
-        g_list_free (languages);
-    }
+    FRAME_NEW (_("Spell Checking"));
+    /* FIXME: Provide a nice dictionary selection */
+    button = katze_property_proxy (settings, "enable-spell-checking", NULL);
+    gtk_button_set_label (GTK_BUTTON (button), _("Enable Spell Checking"));
+    gtk_widget_set_tooltip_text (button, _("Enable spell checking while typing"));
+    INDENTED_ADD (button);
+    button = gtk_label_new (_("Spelling dictionaries:"));
+    INDENTED_ADD (button);
+    entry = katze_property_proxy (settings, "spell-checking-languages", NULL);
+    /* i18n: The example should be adjusted to contain a good local default */
+    gtk_widget_set_tooltip_text (entry, _("A comma separated list of "
+       "languages to be used for spell checking, for example \"en_GB,de_DE\""));
+    SPANNED_ADD (entry);
     #endif
 
     /* Page "Interface" */
@@ -498,7 +473,7 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     SPANNED_ADD (entry);
     label = katze_property_label (settings, "download-manager");
     INDENTED_ADD (label);
-    entry = katze_property_proxy (settings, "download-manager", "application-Network");
+    entry = katze_property_proxy (settings, "download-manager", "application-FileTransfer");
     SPANNED_ADD (entry);
     label = katze_property_label (settings, "news-aggregator");
     INDENTED_ADD (label);
@@ -510,21 +485,27 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     PAGE_NEW (GTK_STOCK_NETWORK, _("Network"));
     FRAME_NEW (_("Network"));
     #if !HAVE_HILDON
-    label = katze_property_label (settings, "http-proxy");
+    label = katze_property_label (settings, "proxy-type");
+    INDENTED_ADD (label);
+    button = katze_property_proxy (settings, "proxy-type", NULL);
+    SPANNED_ADD (button);
+    label = gtk_label_new (_("Hostname"));
     INDENTED_ADD (label);
     entry = katze_property_proxy (settings, "http-proxy", NULL);
     SPANNED_ADD (entry);
-    INDENTED_ADD (gtk_event_box_new ());
-    button = katze_property_proxy (settings, "auto-detect-proxy", NULL);
-    SPANNED_ADD (button);
-    g_signal_connect (settings, "notify::auto-detect-proxy",
-        G_CALLBACK (midori_preferences_notify_auto_detect_proxy_cb), entry);
-    midori_preferences_notify_auto_detect_proxy_cb (settings, NULL, entry);
+    g_signal_connect (settings, "notify::proxy-type",
+        G_CALLBACK (midori_preferences_notify_proxy_type_cb), entry);
+    midori_preferences_notify_proxy_type_cb (settings, NULL, entry);
     #endif
     label = katze_property_label (settings, "identify-as");
     INDENTED_ADD (label);
-    button = katze_property_proxy (settings, "identify-as", "custom-ident-string");
+    button = katze_property_proxy (settings, "identify-as", "custom-user-agent");
     SPANNED_ADD (button);
+    label = katze_property_label (settings, "preferred-languages");
+    INDENTED_ADD (label);
+    entry = katze_property_proxy (settings, "preferred-languages", NULL);
+    SPANNED_ADD (entry);
+
 
     /* Page "Privacy" */
     PAGE_NEW (GTK_STOCK_INDEX, _("Privacy"));
@@ -533,16 +514,21 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     INDENTED_ADD (label);
     button = katze_property_proxy (settings, "accept-cookies", NULL);
     SPANNED_ADD (button);
-    button = katze_property_proxy (settings, "original-cookies-only", NULL);
-    INDENTED_ADD (button);
     label = katze_property_label (settings, "maximum-cookie-age");
     INDENTED_ADD (label);
     entry = katze_property_proxy (settings, "maximum-cookie-age", NULL);
     SPANNED_ADD (entry);
     label = gtk_label_new (_("days"));
     SPANNED_ADD (label);
+    #if WEBKIT_CHECK_VERSION (1, 1, 8)
+    INDENTED_ADD (katze_property_proxy (settings, "enable-html5-database", NULL));
+    SPANNED_ADD (katze_property_proxy (settings, "enable-html5-local-storage", NULL));
+    #endif
+    #if WEBKIT_CHECK_VERSION (1, 1, 13)
+    INDENTED_ADD (katze_property_proxy (settings, "enable-offline-web-application-cache", NULL));
+    #endif
     FRAME_NEW (_("History"));
-    button = katze_property_proxy (settings, "remember-last-visited-pages", NULL);
+    button = katze_property_label (settings, "maximum-history-age");
     INDENTED_ADD (button);
     button = katze_property_proxy (settings, "maximum-history-age", NULL);
     SPANNED_ADD (button);
