@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2008-2009 Christian Dywan <christian@twotoasts.de>
+ Copyright (C) 2008-2010 Christian Dywan <christian@twotoasts.de>
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
@@ -28,67 +28,63 @@ struct _MidoriWebSettings
 {
     WebKitWebSettings parent_instance;
 
-    gboolean remember_last_window_size;
+    gboolean remember_last_window_size : 1;
+    MidoriWindowState last_window_state : 2;
+    gboolean show_menubar : 1;
+    gboolean show_navigationbar : 1;
+    gboolean show_bookmarkbar : 1;
+    gboolean show_panel : 1;
+    gboolean show_transferbar : 1;
+    gboolean show_statusbar : 1;
+    MidoriToolbarStyle toolbar_style : 3;
+    gboolean progress_in_location : 1;
+    gboolean search_engines_in_completion : 1;
+    gboolean compact_sidepanel : 1;
+    gboolean show_panel_controls : 1;
+    gboolean right_align_sidepanel : 1;
+    gboolean open_panels_in_windows : 1;
+    MidoriStartup load_on_startup : 2;
+    gboolean show_crash_dialog : 1;
+    gboolean speed_dial_in_new_tabs : 1;
+    gboolean ask_for_destination_folder : 1;
+    gboolean notify_transfer_completed : 1;
+    MidoriPreferredEncoding preferred_encoding : 3;
+    gboolean always_show_tabbar : 1;
+    gboolean close_buttons_on_tabs : 1;
+    MidoriNewPage open_new_pages_in : 2;
+    MidoriNewPage open_external_pages_in : 2;
+    gboolean middle_click_opens_selection : 1;
+    gboolean open_tabs_in_the_background : 1;
+    gboolean open_tabs_next_to_current : 1;
+    gboolean open_popups_in_tabs : 1;
+    gboolean zoom_text_and_images : 1;
+    gboolean find_while_typing : 1;
+    gboolean kinetic_scrolling : 1;
+    MidoriAcceptCookies accept_cookies : 2;
+    gboolean original_cookies_only : 1;
+    gboolean remember_last_visited_pages : 1;
+    gboolean remember_last_downloaded_files : 1;
+    MidoriProxy proxy_type : 2;
+    gboolean auto_detect_proxy : 1;
+    MidoriIdentity identify_as : 3;
+
     gint last_window_width;
     gint last_window_height;
-    MidoriWindowState last_window_state;
     gint last_panel_position;
     gint last_panel_page;
     gint last_web_search;
+    gint maximum_cookie_age;
+    gint maximum_history_age;
 
-    gboolean show_menubar;
-    gboolean show_navigationbar;
-    gboolean show_bookmarkbar;
-    gboolean show_panel;
-    gboolean show_transferbar;
-    gboolean show_statusbar;
-
-    MidoriToolbarStyle toolbar_style;
-    gboolean progress_in_location;
-    gboolean search_engines_in_completion;
     gchar* toolbar_items;
-    gboolean compact_sidepanel;
-    gboolean show_panel_controls;
-    gboolean right_align_sidepanel;
-    gboolean open_panels_in_windows;
-
-    MidoriStartup load_on_startup;
     gchar* homepage;
-    gboolean show_crash_dialog;
-    gboolean speed_dial_in_new_tabs;
     gchar* download_folder;
-    gboolean ask_for_destination_folder;
-    gboolean notify_transfer_completed;
     gchar* download_manager;
     gchar* text_editor;
     gchar* news_aggregator;
     gchar* location_entry_search;
-    MidoriPreferredEncoding preferred_encoding;
-
-    gboolean always_show_tabbar;
-    gboolean close_buttons_on_tabs;
-    MidoriNewPage open_new_pages_in;
-    MidoriNewPage open_external_pages_in;
-    gboolean middle_click_opens_selection;
-    gboolean open_tabs_in_the_background;
-    gboolean open_tabs_next_to_current;
-    gboolean open_popups_in_tabs;
-
-    gboolean zoom_text_and_images;
-    gboolean find_while_typing;
-    gboolean kinetic_scrolling;
-    MidoriAcceptCookies accept_cookies;
-    gboolean original_cookies_only;
-    gint maximum_cookie_age;
-
-    gboolean remember_last_visited_pages;
-    gint maximum_history_age;
-    gboolean remember_last_downloaded_files;
-
     gchar* http_proxy;
     gchar* http_accept_language;
-    gboolean auto_detect_proxy;
-    MidoriIdentity identify_as;
     gchar* ident_string;
 
     gint clear_private_data;
@@ -170,6 +166,7 @@ enum
     PROP_MAXIMUM_HISTORY_AGE,
     PROP_REMEMBER_LAST_DOWNLOADED_FILES,
 
+    PROP_PROXY_TYPE,
     PROP_HTTP_PROXY,
     PROP_AUTO_DETECT_PROXY,
     PROP_IDENTIFY_AS,
@@ -272,6 +269,23 @@ midori_toolbar_style_get_type (void)
     return type;
 }
 
+GType
+midori_proxy_get_type (void)
+{
+    static GType type = 0;
+    if (!type)
+    {
+        static const GEnumValue values[] = {
+         { MIDORI_PROXY_AUTOMATIC, "MIDORI_PROXY_AUTOMATIC", N_("Automatic (GNOME or environment)") },
+         { MIDORI_PROXY_HTTP, "MIDORI_PROXY_HTTP", N_("HTTP proxy server") },
+         { MIDORI_PROXY_NONE, "MIDORI_PROXY_NONE", N_("No proxy server") },
+         { 0, NULL, NULL }
+        };
+        type = g_enum_register_static ("MidoriProxy", values);
+    }
+    return type;
+}
+
 GType
 midori_accept_cookies_get_type (void)
 {
@@ -1019,22 +1033,39 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
 
 
 
+    /**
+     * MidoriWebSettings:proxy-type:
+     *
+     * The type of proxy server to use.
+     *
+     * Since: 0.2.5
+     */
+    g_object_class_install_property (gobject_class,
+                                     PROP_PROXY_TYPE,
+                                     g_param_spec_enum (
+                                     "proxy-type",
+                                     _("Proxy server"),
+                                     _("The type of proxy server to use"),
+                                     MIDORI_TYPE_PROXY,
+                                     MIDORI_PROXY_AUTOMATIC,
+                                     flags));
+
     g_object_class_install_property (gobject_class,
                                      PROP_HTTP_PROXY,
                                      g_param_spec_string (
                                      "http-proxy",
-                                     _("Proxy Server"),
+                                     _("HTTP Proxy Server"),
                                      _("The proxy server used for HTTP connections"),
                                      NULL,
                                      flags));
 
     /**
-    * MidoriWebSettings:auto-detect-proxy:
-    *
-    * Whether to detect the proxy server automatically from the environment
-    *
-    * Since: 0.1.3
-    */
+     * MidoriWebSettings:auto-detect-proxy:
+     *
+     * Whether to detect the proxy server automatically from the environment
+     *
+     * Deprecated: 0.2.5
+     */
     g_object_class_install_property (gobject_class,
                                      PROP_AUTO_DETECT_PROXY,
                                      g_param_spec_boolean (
@@ -1188,7 +1219,7 @@ get_sys_name (void)
     {
         struct utsname name;
         if (uname (&name) != -1)
-            sys_name = g_strdup_printf ("%s %s", name.sysname, name.machine);
+            sys_name = g_strdup(name.sysname);
         else
             sys_name = "Unix";
     }
@@ -1214,15 +1245,10 @@ generate_ident_string (MidoriIdentity identify_as)
 
     const gchar* os =
     #if HAVE_OSX
-    /* #if defined (HAVE_X86) */
-    "Intel Mac OS X";
-    /* #else
-    "PPC Mac OS X";
-    #endif */
+    "Mac OS X";
     #elif defined (G_OS_UNIX)
     get_sys_name ();
     #elif defined (G_OS_WIN32)
-    // FIXME: Windows NT version
     "Windows";
     #else
     "Unknown";
@@ -1230,8 +1256,7 @@ generate_ident_string (MidoriIdentity identify_as)
 
     const gchar* appname = "Midori/"
         G_STRINGIFY (MIDORI_MAJOR_VERSION) "."
-        G_STRINGIFY (MIDORI_MINOR_VERSION) "."
-        G_STRINGIFY (MIDORI_MICRO_VERSION);
+        G_STRINGIFY (MIDORI_MINOR_VERSION);
 
     const gchar* lang = pango_language_to_string (gtk_get_default_language ());
 
@@ -1497,6 +1522,13 @@ midori_web_settings_set_property (GObject*      object,
         web_settings->remember_last_downloaded_files = g_value_get_boolean (value);
         break;
 
+    case PROP_PROXY_TYPE:
+        web_settings->proxy_type = g_value_get_enum (value);
+        web_settings->auto_detect_proxy =
+            web_settings->proxy_type == MIDORI_PROXY_AUTOMATIC
+            ? TRUE : FALSE;
+        g_object_notify (object, "auto-detect-proxy");
+    break;
     case PROP_HTTP_PROXY:
         katze_assign (web_settings->http_proxy, g_value_dup_string (value));
         break;
@@ -1741,6 +1773,9 @@ midori_web_settings_get_property (GObject*    object,
         g_value_set_boolean (value, web_settings->remember_last_downloaded_files);
         break;
 
+    case PROP_PROXY_TYPE:
+        g_value_set_enum (value, web_settings->proxy_type);
+        break;
     case PROP_HTTP_PROXY:
         g_value_set_string (value, web_settings->http_proxy);
         break;
