@@ -32,6 +32,10 @@ midori_browser_edit_bookmark_dialog_new (MidoriBrowser* browser,
                                          gboolean       new_bookmark,
                                          gboolean       is_folder);
 
+void
+midori_browser_open_bookmark (MidoriBrowser* browser,
+                              KatzeItem*     item);
+
 struct _MidoriBookmarks
 {
     GtkVBox parent_instance;
@@ -42,7 +46,6 @@ struct _MidoriBookmarks
     GtkWidget* treeview;
     MidoriApp* app;
     KatzeArray* array;
-    KatzeNet* net;
 };
 
 struct _MidoriBookmarksClass
@@ -223,7 +226,6 @@ midori_bookmarks_get_toolbar (MidoriViewable* viewable)
         GtkToolItem* toolitem;
 
         toolbar = gtk_toolbar_new ();
-        gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_BOTH_HORIZ);
         gtk_toolbar_set_icon_size (GTK_TOOLBAR (toolbar), GTK_ICON_SIZE_BUTTON);
         bookmarks->toolbar = toolbar;
         toolitem = gtk_tool_button_new_from_stock (GTK_STOCK_ADD);
@@ -563,9 +565,7 @@ midori_bookmarks_treeview_render_icon_cb (GtkTreeViewColumn* column,
         pixbuf = gtk_widget_render_icon (treeview, GTK_STOCK_DIRECTORY,
                                          GTK_ICON_SIZE_MENU, NULL);
     else if (katze_item_get_uri (item))
-        pixbuf = katze_net_load_icon (
-            MIDORI_BOOKMARKS (gtk_widget_get_parent (treeview))->net,
-            katze_item_get_uri (item), NULL, treeview, NULL);
+        pixbuf = katze_load_cached_icon (katze_item_get_uri (item), treeview);
     g_object_set (renderer, "pixbuf", pixbuf, NULL);
 
     if (pixbuf)
@@ -603,19 +603,17 @@ midori_bookmarks_row_activated_cb (GtkTreeView*       treeview,
     GtkTreeModel* model;
     GtkTreeIter iter;
     KatzeItem* item;
-    const gchar* uri;
 
     model = gtk_tree_view_get_model (treeview);
 
     if (gtk_tree_model_get_iter (model, &iter, path))
     {
+        MidoriBrowser* browser;
+
         gtk_tree_model_get (model, &iter, 0, &item, -1);
-        uri = katze_item_get_uri (item);
-        if (uri && *uri)
-        {
-            MidoriBrowser* browser = midori_browser_get_for_widget (GTK_WIDGET (bookmarks));
-            midori_browser_set_current_uri (browser, uri);
-        }
+
+        browser = midori_browser_get_for_widget (GTK_WIDGET (bookmarks));
+        midori_browser_open_bookmark (browser, item);
 
         g_object_unref (item);
     }
@@ -797,8 +795,7 @@ midori_bookmarks_popup (GtkWidget*       widget,
     midori_bookmarks_popup_item (menu, GTK_STOCK_DELETE, NULL,
         item, midori_bookmarks_delete_activate_cb, bookmarks);
 
-    sokoke_widget_popup (widget, GTK_MENU (menu),
-                         event, SOKOKE_MENU_POSITION_CURSOR);
+    katze_widget_popup (widget, GTK_MENU (menu), event, KATZE_MENU_POSITION_CURSOR);
 }
 
 static gboolean
@@ -892,8 +889,6 @@ midori_bookmarks_init (MidoriBookmarks* bookmarks)
     GtkCellRenderer* renderer_pixbuf;
     GtkCellRenderer* renderer_text;
 
-    bookmarks->net = katze_net_new ();
-
     /* Create the treeview */
     model = midori_bookmark_store_new (1, KATZE_TYPE_ITEM);
     treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
@@ -938,7 +933,6 @@ midori_bookmarks_finalize (GObject* object)
     midori_bookmarks_disconnect_folder (bookmarks, bookmarks->array);
     if (bookmarks->app)
         g_object_unref (bookmarks->app);
-    g_object_unref (bookmarks->net);
 }
 
 /**
