@@ -210,9 +210,12 @@ midori_history_read_from_db (MidoriHistory* history,
     {
         gchar* filterstr;
 
-        sqlcmd = "SELECT uri, title, date "
-                 "FROM history WHERE uri LIKE ? or title LIKE ? "
-                 "GROUP BY uri ORDER BY date ASC";
+        sqlcmd = "SELECT uri, title, day FROM history_view "
+                 "WHERE uri LIKE ? or title LIKE ? GROUP BY uri "
+                 "UNION ALL "
+                 "SELECT replace(uri, '%s', title) AS uri, title, day "
+                 "FROM search_view WHERE title LIKE ?1 GROUP BY uri "
+                 "ORDER BY day ASC";
         result = sqlite3_prepare_v2 (db, sqlcmd, -1, &statement, NULL);
         filterstr = g_strdup_printf ("%%%s%%", filter);
         sqlite3_bind_text (statement, 1, filterstr, -1, g_free);
@@ -426,7 +429,6 @@ midori_history_get_toolbar (MidoriViewable* viewable)
         #endif
 
         toolbar = gtk_toolbar_new ();
-        gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_BOTH_HORIZ);
         gtk_toolbar_set_icon_size (GTK_TOOLBAR (toolbar), GTK_ICON_SIZE_BUTTON);
         history->toolbar = toolbar;
         #if HAVE_SQLITE
