@@ -17,7 +17,6 @@
 #include "midori-view.h"
 
 #include "sokoke.h"
-#include "compat.h"
 #include <glib/gi18n.h>
 
 struct _MidoriTransfers
@@ -128,7 +127,6 @@ midori_transfers_get_toolbar (MidoriViewable* transfers)
         GtkToolItem* toolitem;
 
         toolbar = gtk_toolbar_new ();
-        gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_BOTH_HORIZ);
         gtk_toolbar_set_icon_size (GTK_TOOLBAR (toolbar), GTK_ICON_SIZE_BUTTON);
         toolitem = gtk_tool_item_new ();
         gtk_toolbar_insert (GTK_TOOLBAR (toolbar), toolitem, -1);
@@ -483,8 +481,7 @@ midori_transfers_popup (GtkWidget*       widget,
         _("Copy Link Loc_ation"), download,
         midori_transfers_copy_address_activate_cb, FALSE, transfers);
 
-    sokoke_widget_popup (widget, GTK_MENU (menu),
-                         event, SOKOKE_MENU_POSITION_CURSOR);
+    katze_widget_popup (widget, GTK_MENU (menu), event, KATZE_MENU_POSITION_CURSOR);
 }
 #endif
 
